/**
 * Copyright 2001-2007 The eXo Platform SAS          All rights reserved.
 * Please look at license.txt in info directory for more license detail.
 **/

/**
 * Created by The eXo Platform SAS 
 * Author : Mestrallet Benjamin
 *          benjmestrallet@users.sourceforge.net
 * Date: Aug 20, 2003
 * Time: 2:00:32 PM
 */
package org.exoplatform.services.portletcontainer.plugins.pc.portletAPIImp.tags;

import javax.servlet.ServletRequest;
import javax.servlet.jsp.tagext.BodyTagSupport;
import javax.servlet.jsp.JspException;
import javax.portlet.*;

import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.exoplatform.services.portletcontainer.plugins.pc.portletAPIImp.BaseURLImp;

public abstract class XURLTag extends BodyTagSupport {

  protected WindowState windowState;
  protected PortletMode portletMode;
  protected String var;
  protected boolean secure;
  private Map parameters = new HashMap();
  private Map properties = new HashMap();
  protected boolean copyCurrentRenderParameters = false;
  protected boolean escapeXml = true;
  protected boolean changedEscapeXml = false;
  protected String resourceID;
  protected String resourceCacheability;

  public void addParameter(String key, String value){
    parameters.put(key, new String[]{value});    
  }

  public void addProperty(String key, String value){
    properties.put(key, new String[]{value});    
  }

  public void setProperties(BaseURL url, Map map) {
    if(map == null)
      return;
    if(map.containsKey(null))
      return;
    Set keys = map.keySet();
    for (Iterator iter = keys.iterator(); iter.hasNext();) {
      String name = (String) iter.next();
      Object value = map.get(name);
      if (value instanceof String)
        url.addProperty(name, (String) value);
      else if (value instanceof String[]) {
        String[] saVal = (String[]) value;
        for (String sVal : saVal)
          url.addProperty(name, sVal);
      }
    }
  }

  public void setWindowState(String windowState) {
    if(WindowState.MAXIMIZED.toString().equals(windowState))
      this.windowState = WindowState.MAXIMIZED;
    else if (WindowState.MINIMIZED.toString().equals(windowState))
      this.windowState = WindowState.MINIMIZED;
    else if (WindowState.NORMAL.toString().equals(windowState))
      this.windowState = WindowState.NORMAL;
    else
      this.windowState = new WindowState(windowState);  
  }

  public void setPortletMode(String portletMode) {
    if(PortletMode.EDIT.toString().equals(portletMode))
      this.portletMode = PortletMode.EDIT;
    else if(PortletMode.HELP.toString().equals(portletMode))
      this.portletMode = PortletMode.HELP;
    else if(PortletMode.VIEW.toString().equals(portletMode))
      this.portletMode = PortletMode.VIEW;
    else
      this.portletMode = new PortletMode(portletMode);  
  }

  public void setVar(String var) {
    this.var = var;
  }

  public void setSecure(String secure) {
    if(secure.equals("true"))
      this.secure = true;
    else
      this.secure = false;
  }
  
  public void setCopyCurrentRenderParameters(boolean copyCurrentRenderParameters) {
    this.copyCurrentRenderParameters = copyCurrentRenderParameters;
  }
  
  public void setEscapeXml(boolean escapeXml) {
    this.escapeXml = escapeXml;
    this.changedEscapeXml = true;
  }
  
  public void setResourceID(String resourceID) {
    this.resourceID = resourceID;
  }
  
  public void setResourceCacheability(String resourceCacheability) {
    this.resourceCacheability = resourceCacheability;
  }
  
  public int doStartTag() throws JspException {
    parameters = new HashMap();
    return EVAL_BODY_BUFFERED;
  }

  public int doEndTag() throws JspException {
    BaseURL baseURL = (BaseURL)getPortletURL();
    baseURL.setParameters(parameters);
    setProperties(baseURL, properties);
    try {
      baseURL.setSecure(secure);
      if (baseURL instanceof PortletURL) {
        PortletURL portletURL = (PortletURL) baseURL;
        if(portletMode != null)
          portletURL.setPortletMode(portletMode);        
        if(windowState != null)
          portletURL.setWindowState(windowState);
        if (portletURL instanceof ResourceURL) {
          ((ResourceURL)portletURL).setResourceID(resourceID);
          ((ResourceURL)portletURL).setCacheability(resourceCacheability);
        } else {
          // Otherwise action or render URL
          if (copyCurrentRenderParameters) {
            ServletRequest request =  pageContext.getRequest();
            PortletRequest portletRequest = (PortletRequest) request.getAttribute("javax.portlet.request") ;
            baseURL.setParameters(portletRequest.getParameterMap());
          }
        }
      }
    } catch (PortletModeException e) {
      throw new JspException(e);
    } catch (WindowStateException e) {
      throw new JspException(e);
    } catch (PortletSecurityException e) {
      throw new JspException(e);
    }

    if(var == null || "".equals(var)){
      try {        
        pageContext.getOut().print( URLToString(baseURL) );
      } catch (IOException e1) {
        throw new JspException(e1);        
      }
    } else {
      pageContext.setAttribute(var, URLToString(baseURL) );
    }
    return EVAL_PAGE;
  }

  public String URLToString(BaseURL baseURL){
    if (changedEscapeXml)
      return ((BaseURLImp)baseURL).toString(escapeXml);
    else 
      return ((BaseURLImp)baseURL).toString();
  }
  
  
  public abstract BaseURL getPortletURL();
  
}
