/**
 * Copyright 2001-2007 The eXo Platform SAS         All rights reserved. 
 * Please look at license.txt in info directory for more license detail.  
 */

package org.exoplatform.services.wsrp.producer.impl.helpers;


import java.util.List;
import java.util.Set;
import java.net.URLEncoder;
import java.util.Iterator;

import javax.portlet.ResourceURL;

import org.exoplatform.Constants;
import org.exoplatform.commons.utils.IdentifierUtil;
import org.exoplatform.services.portletcontainer.plugins.pc.portletAPIImp.ResourceURLImp;
import org.exoplatform.services.wsrp.WSRPConstants;
import org.exoplatform.services.wsrp.exceptions.WSRPException;
import org.exoplatform.services.wsrp.producer.PersistentStateManager;

/**
 * @author  Mestrallet Benjamin
 *          benjmestrallet@users.sourceforge.net
 */
public class ConsumerRewriterResourceURLImp extends ResourceURLImp{

  private String sessionID;
  private String portletHandle;
  private String template;
  private PersistentStateManager stateManager;

  public ConsumerRewriterResourceURLImp(String type, String markup,
                      List supports,
                      boolean isCurrentlySecured,
                      String template, String portletHandle,
                      PersistentStateManager stateManager,
                      String sessionID){
    super(type, markup, isCurrentlySecured, template, null, true, ResourceURL.PAGE);
    this.template = template;
    this.portletHandle = portletHandle;
    this.stateManager = stateManager;
    this.sessionID = sessionID;
  }

  public String toString() {
    if(!setSecureCalled && isCurrentlySecured){
      isSecure = true;
    }

    String navigationalState = IdentifierUtil.generateUUID(this);
    try {
      stateManager.putNavigationalState(navigationalState, parameters);
    } catch (WSRPException e) {
      e.printStackTrace();
    }

    StringBuffer sB = new StringBuffer();
    sB.append(template);
    sB.append("&");
    sB.append(WSRPConstants.WSRP_URL_TYPE);
    sB.append("=");
    sB.append(type);
    sB.append("&");
    sB.append(WSRPConstants.WSRP_PORTLET_HANDLE);
    sB.append("=");
    sB.append(portletHandle);
    sB.append("&");
    sB.append(WSRPConstants.WSRP_NAVIGATIONAL_STATE);
    sB.append("=");
    sB.append(navigationalState);
    sB.append("&");
    sB.append(WSRPConstants.WSRP_SESSION_ID);
    sB.append("=");
    sB.append(sessionID);
    sB.append("&");
    sB.append(WSRPConstants.WSRP_SECURE_URL);
    sB.append("=");
    sB.append(isSecure);

    if (resourceID != null) {
      sB.append("&");
      sB.append(WSRPConstants.WSRP_RESOURCE_ID);
      sB.append("=");
      sB.append(resourceID);
    }
    if (cacheLevel != null) {
      sB.append("&");
      sB.append(WSRPConstants.WSRP_CACHELEVEL);
      sB.append("=");
      sB.append(cacheLevel);
    }
    sB.append(WSRPConstants.WSRP_REWRITE_SUFFFIX);

    Set names = parameters.keySet();
    for (Iterator iterator = names.iterator(); iterator.hasNext();) {
      String name = (String) iterator.next() ;
      Object obj =  parameters.get(name) ;
      if (obj instanceof String) {
      String value = (String) obj ;
      sB.append(Constants.AMPERSAND);
      sB.append(URLEncoder.encode(name));
      sB.append("=");
      sB.append(URLEncoder.encode(value));
      } else {
        String[] values = (String[]) obj ;
        for (int i=0; i < values.length ; i++) {
          sB.append(Constants.AMPERSAND);
          sB.append(URLEncoder.encode(name));
          sB.append("=");
          sB.append(URLEncoder.encode(values[i]));
        }
      }
    }

    return sB.toString();
  }

  
}
